#include <hex.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

extern int help_page;
extern off_t text_size;
extern byte *text;
extern byte *fname;
extern int modified;
extern off_t cursor;

static int write_data(byte *name, byte *data, off_t len);

byte *read_data(byte *name, off_t *sizebuf, byte *oldbuf, off_t max)
{
	int fd;
	off_t size, i;
	byte *tmp;

	fd = open(name, O_RDONLY);
	if (fd == -1) {
		complain(!max, "Can't open file %s for reading: %s",
			name, strerror(errno));
		return NULL;
	}
	size = lseek(fd, 0, SEEK_END);
	if (max && (size > max)) size = max;
	lseek(fd, 0, SEEK_SET);

	if (!size) {
		close(fd);
		complain(!max, "Refusing to read empty file %s", name);
		return NULL;
	}

	if (oldbuf) {
		tmp = oldbuf;
	} else {
		tmp = (byte *) malloc(size);
		if (!tmp) {
			close(fd);
			complain(1, "Can't open file %s for reading: %s",
				name, strerror(errno));
			return NULL;
		}
	}
	i = read(fd, tmp, size);
	close(fd);
	if (i != size) {
		free(tmp);
		complain(!max, "Can't open file %s for reading: "
			"incomplete file", name);
		return NULL;
	}
	if (sizebuf) *sizebuf = size;
	return tmp;
}

static int write_data(byte *name, byte *data, off_t len)
{
	int fd;
	char tmp[256];

	if (!*name) return -1;
	sprintf(tmp, "%s~", name);
	rename(name, tmp);
	fd = open(name, O_WRONLY | O_CREAT | O_TRUNC,
		S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
	if (fd == -1) {
		complain(0, "Error saving file %s: %s",
			name, strerror(errno));
		return -1;
	}
	write(fd, data, len);
	close(fd);
	complain(0, "File %s saved.", name);
	return 0;
}
	
void save_file(void)
{
	char tmp[256];

	if (input_status("Name of file to save (^C to abort): ", tmp, 255))
		return;
	if (!write_data(tmp, text, text_size)) {
		modified = 0; show_top(modified);
	}
	move_cursor(cursor);
}

void save_block(int s, int e)
{
	char tmp[256];

	if (s >= text_size) s = text_size;
	if (e >= text_size) e = text_size;
	if (s > e) return;

	if (input_status("Name of file to write (^C to abort): ", tmp, 255))
		return;
	write_data(tmp, text + s, e - s + 1);
	move_cursor(cursor);
}

void insert_file(void)
{
	char tmp[256];

	if (input_status("Name of file to insert (^C to abort): ", tmp, 255))
		return;

	/* don't worry about errors, read_data handles those */
	if (read_data(tmp, NULL, text + cursor, text_size - cursor)) {
		modified = 1; show_top(modified);
	}
	update_screen(0);
	move_cursor(cursor);
}

/* save'n'exit */
void snx(void)
{
	if (modified) {
		write_data(fname, text, text_size);
	} else {
		complain(0, "File %s not changed so no update needed.\n",
			fname);
	}
	clean_exit();
}

void edit_file(void)
{
	char tmp[256];

	if (input_status("Name of file to edit (^C to abort): ", tmp, 255))
		return;

	if (text) free(text);

	/* don't worry about errors, read_data handles those */
	text = read_data(tmp, &text_size, NULL, 0);
	modified = 0;
	show_top(modified);
	update_screen(1);
	move_cursor(cursor);
}
